@@ -21,20 +21,20 @@ import os
 import sys
 import warnings
 
-from sumatra.dependency_finder import core, neuron, python, genesis
+from sumatra.dependency_finder import core, neuron, python, genesis, matlab
     
 
 def find_dependencies(filename, executable):
     """Return a list of dependencies for a given script and programming
        language."""
-    if executable.name == "Python":
-        return python.find_dependencies(filename, executable.path)
+    if "matlab" in executable.name.lower():
+        return matlab.find_dependencies(filename, executable)
+    elif "python" in executable.name.lower():
+        return python.find_dependencies(filename, executable)
     elif executable.name == "NEURON":
-        return neuron.find_dependencies(filename, executable.path)
+        return neuron.find_dependencies(filename, executable)
     elif executable.name == "GENESIS":
-        return genesis.find_dependencies(filename, executable.path)
+        return genesis.find_dependencies(filename, executable)
     else:
         warnings.warn("find_dependencies() not yet implemented for %s" % executable.name)
-        return []
-        
-
+        return []
\ No newline at end of file
