@@ -9,6 +9,18 @@ import shelve
 from datetime import datetime
 
 
+def check_name(f):
+    """
+    Some backends to shelve do not accept unicode variables as keys.
+    This decorator therefore converts a unicode project_name to a string
+    before calling the wrapped method. See http://bugs.python.org/issue1036490
+    """
+    def wrapped(self, project_name, *args):
+        project_name = str(project_name)
+        return f(self, project_name, *args)
+    return wrapped
+
+
 class ShelveRecordStore(RecordStore):
     
     def __init__(self, shelf_name=".smt/records"):
@@ -31,6 +43,7 @@ class ShelveRecordStore(RecordStore):
     def list_projects(self):
         return self.shelf.keys()
 
+    @check_name
     def save(self, project_name, record):
         if self.shelf.has_key(project_name):
             records = self.shelf[project_name]
@@ -38,10 +51,12 @@ class ShelveRecordStore(RecordStore):
             records = {}
         records[record.label] = record
         self.shelf[project_name] = records
-        
+    
+    @check_name
     def get(self, project_name, label):
         return self.shelf[project_name][label]
     
+    @check_name
     def list(self, project_name, tags=None):
         if project_name in self.shelf:
             if tags:
@@ -57,23 +72,27 @@ class ShelveRecordStore(RecordStore):
             records = []
         return records
     
+    @check_name
     def labels(self, project_name):
         if project_name in self.shelf:
             return self.shelf[project_name].keys()
         else:
             return []
     
+    @check_name
     def delete(self, project_name, label):
         records = self.shelf[project_name]
         records.pop(label)
         self.shelf[project_name] = records
-        
+    
+    @check_name    
     def delete_by_tag(self, project_name, tag):
         for_deletion = [record for record in self.shelf[project_name].values() if tag in record.tags]
         for record in for_deletion:
             self.delete(project_name, record.label)
         return len(for_deletion)
-        
+    
+    @check_name    
     def most_recent(self, project_name):
         most_recent = None
         most_recent_timestamp = datetime.min
