@@ -27,10 +27,15 @@ except ImportError:
     from mercurial.dispatch import _findrepo as findrepo
 import os
 import binascii
+import functools
 from base import VersionControlError
 
 from base import Repository, WorkingCopy
 
+def vectorized(generator_func):
+    def wrapper(*args, **kwargs):
+        return list(generator_func(*args, **kwargs))
+    return functools.update_wrapper(wrapper, generator_func)
 
 def may_have_working_copy(path=None):
     path = path or os.getcwd()
@@ -43,8 +48,13 @@ def get_working_copy(path=None):
     else:
         raise VersionControlError("No Mercurial working copy found at %s" % path)
 
+
 def get_repository(url):
-    return MercurialRepository(url)
+    repos = MercurialRepository(url)
+    if repos.exists:
+        return repos
+    else:
+        raise VersionControlError("Cannot access Mercurial repository at %s" % self.url)    
 
 
 class MercurialWorkingCopy(WorkingCopy):
@@ -70,14 +80,15 @@ class MercurialWorkingCopy(WorkingCopy):
         hg.update(self.repository._repository, 'tip')
 
     def status(self):
-        modified, added, removed, deleted, unknown, ignored, clean = self.repository._repository.status() #unknown=True)
-        return {'modified': modified, 'removed': removed,
-                'deleted': deleted, 'unknown': unknown}
+        modified, added, removed, missing, unknown, ignored, clean = self.repository._repository.status(ignored=True, clean=True, unknown=True)
+        return {'modified': set(modified), 'removed': set(removed),
+                'missing': set(missing), 'unknown': set(unknown),
+                'clean': set(clean), 'added': set(added)}
 
     def has_changed(self):
         status = self.status()
         changed = False
-        for st in 'modified', 'removed', 'deleted':
+        for st in 'modified', 'removed', 'missing':
             if status[st]:
                 changed = True
                 break
@@ -89,17 +100,41 @@ class MercurialWorkingCopy(WorkingCopy):
         diff = patch.diff(self.repository._repository, opts=opts)
         return "".join(diff)
 
+    def content(self, hex):
+        repo = hg.repository(ui.ui(), self.path)
+        i = 1
+        if hex in repo.parents()[0].hex():
+            ctx = repo.parents()[0] 
+            return ctx.filectx(ctx.files()[0]).data()
+        while True:
+            el = repo.parents(i)[0].hex()
+            if hex in el:  
+                ctx = repo.parents(i)[0]      
+                return ctx.filectx(ctx.files()[0]).data() # presume that we have only one file [0]
+            i += 1
+
 
 class MercurialRepository(Repository):
     
     def __init__(self, url):
         Repository.__init__(self, url)
         self._ui = ui.ui()  # get a ui object
-        try:
-            self._repository = hg.repository(self._ui, url)
-        # need to add a check that this actually is a Mercurial repository
-        except (RepoError, Exception), err:
-            raise VersionControlError("Cannot access repository at %s: %s" % (url, err))
+        self.__repository = None
+
+    @property
+    def exists(self):
+        if self._repository:
+            return True
+
+    @property
+    def _repository(self):
+        if self.__repository is None:
+            try:
+                self.__repository = hg.repository(self._ui, self.url)
+                # need to add a check that this actually is a Mercurial repository
+            except (RepoError, Exception), err:
+                raise VersionControlError("Cannot access Mercurial repository at %s: %s" % (self.url, err))    
+        return self.__repository    
 
     def checkout(self, path="."):
         """Clone a repository."""
@@ -108,11 +143,12 @@ class MercurialRepository(Repository):
             # update
             hg.update(self._repository, None)
         else:
-            # can't clone into an existing directory, so we create
-            # an empty repository and then do a pull and update
-            local_repos = hg.repository(self._ui, path, create=True)
-            local_repos.pull(self._repository)
-            hg.update(local_repos, None)
+            try:
+                hg.clone(self._ui, {}, self.url, path, update=True)
+            except:  # hg.clone fails for older versions of mercurial, e.g. 1.5
+                local_repos = hg.repository(self._ui, path, create=True)
+                local_repos.pull(self._repository)
+                hg.update(local_repos, None)
 
     def get_working_copy(self, path=None):
         return get_working_copy(path)
\ No newline at end of file
