@@ -33,6 +33,7 @@ class Repository(object):
         """For unpickling"""
         self.__init__(state['url'])
 
+
 class WorkingCopy(object):
     
     def __eq__(self, other):
@@ -42,4 +43,11 @@ class WorkingCopy(object):
     def __ne__(self, other):
         return not self.__eq__(other)
 
-
+    def contains(self, path):
+        """
+        Does the repository contain the file with the given path?
+        
+        where `path` is relative to the working copy root.
+        """
+        status = self.status()    
+        return (path in status['modified']) or (path in status['clean'])
