@@ -6,12 +6,13 @@ import unittest
 from datetime import datetime
 from sumatra.records import Record
 from sumatra.formatting import Formatter, TextFormatter, HTMLFormatter, TextDiffFormatter, get_formatter
+from sumatra.core import TIMESTAMP_FORMAT
 from xml.etree import ElementTree
 
 class MockRecord(Record):
     def __init__(self):
         self.timestamp = datetime.now()
-        self.label = self.timestamp.strftime("%Y%m%d-%H%M%S")
+        self.label = self.timestamp.strftime(TIMESTAMP_FORMAT)
         self.reason = "determine how many shekels the gourd is worth"
         self.outcome = "apparently it is worth NaN shekels"
         self.duration = 1.2345
@@ -24,6 +25,8 @@ class MockRecord(Record):
         self.input_data = ['somefile', 'anotherfile']
         self.diff = 'iuefciaeufhmc'
         self.parameters = {'a': 2, 'b': 4}
+        self.launch_mode = 'serial'
+        self.output_data = []
 
 class MockRecordDifference(object):
     recordA = MockRecord()
@@ -43,33 +46,33 @@ class MockRecordDifference(object):
     parameters_differ = True
     input_data_differ = True
     script_arguments_differ = True
-    
+
 
 class TestTextFormatter(unittest.TestCase):
-    
+
     def setUp(self):
         self.record_list = [ MockRecord(), MockRecord() ]
         self.record_tuple = ( MockRecord(), MockRecord() )
-    
+
     def test__init__should_accept_an_iterable_containing_records(self):
         tf1 = TextFormatter(self.record_list)
         tf2 = TextFormatter(self.record_tuple)
-    
+
     def test__format__should_call_the_appropriate_method(self):
         tf1 = TextFormatter(self.record_list)
         self.assertEqual(tf1.format(mode='short'), tf1.short())
         self.assertEqual(tf1.format(mode='long'), tf1.long())
         self.assertEqual(tf1.format(mode='table'), tf1.table())
-        
+
     def test__format__should_raise_an_Exception_with_invalid_mode(self):
         tf1 = TextFormatter(self.record_list)
         self.assertRaises(AttributeError, tf1.format, "foo")
-    
+
     def test__short__should_return_a_multi_line_string(self):
         tf1 = TextFormatter(self.record_list)
         txt = tf1.short()
         self.assertEqual(len(txt.split("\n")), len(tf1.records))
-    
+
     def test__long__should_return_a_fixed_width_string(self):
         tf1 = TextFormatter(self.record_list)
         txt = tf1.long()
@@ -85,11 +88,11 @@ class TestTextFormatter(unittest.TestCase):
 
 
 class TestHTMLFormatter(unittest.TestCase):
-    
+
     def setUp(self):
         self.record_list = [ MockRecord(), MockRecord() ]
         self.record_tuple = ( MockRecord(), MockRecord() )
-        
+
     def test__short__should_return_an_unordered_list(self):
         hf1 = HTMLFormatter(self.record_list)
         doc = ElementTree.fromstring(hf1.short())
@@ -115,23 +118,23 @@ class TestHTMLFormatter(unittest.TestCase):
 
 
 class TestTextDiffFormatter(unittest.TestCase):
-    
+
     def setUp(self):
         self.df = TextDiffFormatter(MockRecordDifference())
-    
+
     def test__init(self):
         pass
-        
+
     def test__short(self):
         txt = self.df.short()
-        
+
     def test__long(self):
         txt = self.df.long()
-    
+
 
 
 class TestModuleFunctions(unittest.TestCase):
-    
+
     def test__get_formatter__should_return_Formatter_subclass(self):
         for format in 'text', 'html', 'textdiff':
             assert issubclass(get_formatter(format), Formatter)
