@@ -3,14 +3,24 @@ Unit tests for the sumatra.launch module
 """
 
 from __future__ import with_statement
-import unittest
+try:
+    import unittest2 as unittest
+except ImportError:
+    import unittest
 from sumatra.launch import SerialLaunchMode, DistributedLaunchMode, PlatformInformation
 import sys
 import os
 
+try:
+    import mpi4py.MPI
+    have_mpi = True
+except ImportError:
+    have_mpi = False
 
 class MockExecutable(object):
+    requires_script = True
     def __init__(self, path):
+        self.name = "foo"
         self.path = path
         self.options = "-t"
 
@@ -90,6 +100,7 @@ class TestSerialLaunchMode(unittest.TestCase, BaseTestLaunchMode):
 
 class TestDistributedLaunchMode(unittest.TestCase, BaseTestLaunchMode):
     
+    @unittest.skipUnless(have_mpi, "Need to install mpi4py")
     def setUp(self):
         self.lm = DistributedLaunchMode(2, "mpiexec", ["node1", "node2"])
 
