@@ -4,16 +4,18 @@ Unit tests for the sumatra.parameters module
 from __future__ import with_statement
 import unittest
 import os
+from copy import deepcopy
 try:
     import json
 except ImportError:
     import simplejson as json
 from textwrap import dedent
-from sumatra.parameters import SimpleParameterSet, NTParameterSet, ConfigParserParameterSet, build_parameters
+from sumatra.parameters import SimpleParameterSet, JSONParameterSet, \
+        NTParameterSet, ConfigParserParameterSet, build_parameters
 
 
 class TestNTParameterSet(unittest.TestCase):
-    
+
     def test__json_should_be_accepted(self):
         example = {
                     "y": {
@@ -23,7 +25,8 @@ class TestNTParameterSet(unittest.TestCase):
                     },
                     "x": 2.9,
                     "z": 100,
-                    "mylabel": "camelot"
+                    "mylabel": "camelot",
+                    "have_horse": False,
         }
         P = NTParameterSet(json.dumps(example))
         self.assertEqual(P.y.a, -2)
@@ -32,24 +35,24 @@ class TestNTParameterSet(unittest.TestCase):
         self.assertEqual(P.mylabel, "camelot")
 
 class TestSimpleParameterSet(unittest.TestCase):
-    
+
     def test__init__should_accept_space_around_equals(self):
         P = SimpleParameterSet("x = 2\ny = 3")
         self.assertEqual(P["x"], 2)
         self.assertEqual(P["y"], 3)
-    
+
     def test__init__should_accept_no_space_around_equals(self):
         P = SimpleParameterSet("x=2")
         self.assertEqual(P["x"], 2)
-        
+
     def test__init__should_accept_hash_as_comment_character(self):
         P = SimpleParameterSet("x = 2 # this is a comment")
         self.assertEqual(P["x"], 2)
-        
+
     def test__init__should_accept_an_empty_initializer(self):
         P = SimpleParameterSet("")
         self.assertEqual(P.values, {})
-        
+
     def test__init__should_accept_a_filename_or_string(self):
         init = "x = 2\ny = 3"
         P1 = SimpleParameterSet(init)
@@ -58,51 +61,55 @@ class TestSimpleParameterSet(unittest.TestCase):
         P2 = SimpleParameterSet("test_file")
         self.assertEqual(P1.values, P2.values)
         os.remove("test_file")
-        
+
     def test__init__should_raise_a_TypeError_if_initializer_is_not_a_filename_or_string(self):
         self.assertRaises(TypeError, SimpleParameterSet, [])
-    
+
     def test__init__should_ignore_empty_lines(self):
         init = "x = 2\n\n\r   \ny = 3\n\n\r\t\t  \n"
         P = SimpleParameterSet(init)
         self.assertEqual(P["x"], 2)
         self.assertEqual(P["y"], 3)
-        
+
     def test__init__should_ignore_comment_lines(self):
         init = "#some parameters\nx = 2\n# this is a comment at column 0\n  # this is an indented comment\n  y = 3"
         P = SimpleParameterSet(init)
         self.assertEqual(P["x"], 2)
         self.assertEqual(P["y"], 3)
-        
+
     def test__init__should_raise_syntaxerror_if_line_doesnt_contain_param_or_comment(self):
         init = "# some data\n1.0 2.0 3.0\n4.0 5.0 6.0"
         self.assertRaises(SyntaxError, SimpleParameterSet, init)
-    
+
+    def test__string_parameters_should_be_able_to_contain_equals_signs(self):
+        init = 'equation = "e = mc^2"'
+        P = SimpleParameterSet(init)
+
     def test__getitem__should_give_access_to_parameters(self):
         P = SimpleParameterSet("x = 2\ny = 3")
         self.assertEqual(P["x"], 2)
         self.assertEqual(P["y"], 3)
-        
+
     def test__getitem__should_raise_a_KeyError_for_a_nonexistent_parameter(self):
         P = SimpleParameterSet("x = 2\ny = 3")
         self.assertRaises(KeyError, P.__getitem__, "z")
-        
+
     def test__pretty__should_put_quotes_around_string_parameters(self):
         init = 'y = "hello"'
         P = SimpleParameterSet(init)
         self.assertEqual(P.pretty(), init)
-        
+
     def test__pretty__should_recreate_comments_in_the_initializer(self):
         init = 'x = 2 # this is a comment'
         P = SimpleParameterSet(init)
         self.assertEqual(P.pretty(), init)
-        
+
     def test__pretty__output_should_be_useable_to_create_an_identical_parameterset(self):
         init = "x = 2\ny = 3\nz = 'hello'"
         P1 = SimpleParameterSet(init)
         P2 = SimpleParameterSet(P1.pretty())
         self.assertEqual(P1.values, P2.values)
-        
+
     def test__save__should_backup_an_existing_file_before_overwriting_it(self):
         # not really sure what the desired behaviour is here
         assert not os.path.exists("test_file.orig")
@@ -114,7 +121,7 @@ class TestSimpleParameterSet(unittest.TestCase):
         self.assert_(os.path.exists("test_file.orig"))
         os.remove("test_file")
         os.remove("test_file.orig")
-        
+
     def test__update__should_only_accept_numbers_or_strings(self):
         # could maybe allow lists of numbers or strings
         P = SimpleParameterSet("x = 2\ny = 3")
@@ -123,25 +130,25 @@ class TestSimpleParameterSet(unittest.TestCase):
         P.update({"tumoltuae": 42})
         self.assertEqual(P["tumoltuae"], 42)
         self.assertRaises(TypeError, P.update, "bar", {})
-    
+
 
 class TestConfigParserParameterSet(unittest.TestCase):
     test_parameters = dedent("""
         # this is a comment
-        
+
         [sectionA]
         a: 2
         b: 3
-        
+
         [sectionB]
         c: hello
         d: world
         """)
-    
+
     def test__init__should_accept_an_empty_initializer(self):
         P = ConfigParserParameterSet("")
         self.assertEqual(P.as_dict(), {})
-        
+
     def test__init__should_accept_a_filename_or_string(self):
         init = self.__class__.test_parameters
         P1 = ConfigParserParameterSet(init)
@@ -156,7 +163,7 @@ class TestConfigParserParameterSet(unittest.TestCase):
         P1 = ConfigParserParameterSet(init)
         P2 = ConfigParserParameterSet(P1.pretty())
         self.assertEqual(P1.as_dict(), P2.as_dict())
-        
+
     def test__update_should_handle_dots_appropriately(self):
         init = self.__class__.test_parameters
         P = ConfigParserParameterSet(init)
@@ -165,9 +172,52 @@ class TestConfigParserParameterSet(unittest.TestCase):
                                        "sectionB": dict(c="hello", d="world"),
                                        "sectionC": dict(e="9")})
 
+    def test__deepcopy(self):
+        # see ticket:93
+        init = self.__class__.test_parameters
+        P = ConfigParserParameterSet(init)
+        Q = deepcopy(P)
+
+
+class TestJSONParameterSet(unittest.TestCase):
+    test_parameters = dedent("""
+        {
+            "a" : 2,
+            "b" : "hello",
+            "c" : {"a":1, "b":2},
+            "d" : [1, 2, 3, 4]
+        }
+        """)
+
+    def test__init__should_accept_an_empty_initializer(self):
+        P = JSONParameterSet("")
+        self.assertEqual(P.as_dict(), {})
+
+    def test__init__should_accept_a_filename_or_string(self):
+        init = self.__class__.test_parameters
+        P1 = JSONParameterSet(init)
+        with open("test_file", "w") as f:
+            f.write(init)
+        P2 = JSONParameterSet("test_file")
+        self.assertEqual(P1.as_dict(), P2.as_dict())
+        os.remove("test_file")
+
+    def test_save(self):
+        init = self.__class__.test_parameters
+        P1 = JSONParameterSet(init)
+        P1.save("test_file")
+        P2 = JSONParameterSet("test_file")
+        self.assertEqual(P1.as_dict(), P2.as_dict())
+        os.remove("test_file")
+
+    def test__pretty__output_should_be_useable_to_create_an_identical_parameterset(self):
+        init = self.__class__.test_parameters
+        P1 = JSONParameterSet(init)
+        P2 = JSONParameterSet(P1.pretty())
+        self.assertEqual(P1.as_dict(), P2.as_dict())
 
 class TestModuleFunctions(unittest.TestCase):
-    
+
     def setUp(self):
         init = "x = 2\ny = 3"
         with open("test_file", "w") as f:
@@ -178,12 +228,12 @@ class TestModuleFunctions(unittest.TestCase):
         init3 = "[sectionA]\na = 2\nb = 3\n[sectionB]\nc = hello\nd = world"
         with open("test_file3", "w") as f:
             f.write(init3)
-    
+
     def tearDown(self):
         os.remove("test_file")
         os.remove("test_file2")
         os.remove("test_file3")
-        
+
 # these tests should now be applied to commands.parse_arguments and/or commands.parse_command_line_parameter()
     #def test__build_parameters__should_add_new_command_line_parameters_to_the_file_parameters(self):
     #    P = build_parameters("test_file", ["z=42", "foo=bar"])
@@ -191,13 +241,13 @@ class TestModuleFunctions(unittest.TestCase):
     #    P = build_parameters("test_file3", ["sectionA.c=42", "sectionC.foo=bar"])
     #    self.assertEqual(P["sectionA.c"], "42")
     #    self.assertEqual(P["sectionC.foo"], "bar")
-    #            
+    #
     #def test__build_parameters__should_overwrite_file_parameters_if_command_line_parameters_have_the_same_name(self):
     #    P = build_parameters("test_file", ["x=12", "y=13"])
     #    self.assertEqual(P.values, {"x": 12, "y": 13})
     #    P = build_parameters("test_file3", ["sectionA.a=999"])
     #    self.assertEqual(P["sectionA.a"], "999")
-    #    
+    #
     #def test__build_parameters__should_insert_dotted_parameters_in_the_appropriate_place_in_the_hierarchy(self):
     #    P = build_parameters("test_file2", ["z=100", "y.c=5", "y.a=-2"])
     #    assert isinstance(P, NTParameterSet), type(P)
@@ -210,15 +260,15 @@ class TestModuleFunctions(unittest.TestCase):
     #                                    "x": 2,
     #                                    "z": 100,
     #                                  })
-    #    
-    #    
+    #
+    #
     #def test__build_parameters__should_cast_string_representations_of_numbers_within_lists_to_numeric_type(self):
     #    # unless they are in quotes
     #    # also applies to tuples
     #    # what about dicts or sets?
     #    P = build_parameters("test_file", ["M=[1,2,3,4,5]", "N=['1', '2', 3, 4, '5']"])
     #    self.assertEqual(P.values, {"x": 2, "y": 3, "M": [1,2,3,4,5], "N": ['1', '2', 3, 4, '5']})
-    
+
 
 
 if __name__ == '__main__':
