@@ -4,7 +4,7 @@ Unit tests for the sumatra.projects module
 
 from __future__ import with_statement
 import shutil
-import os
+import os, sys
 import unittest
 import sumatra.projects
 from sumatra.projects import Project, load_project
@@ -18,14 +18,17 @@ class MockDiffFormatter(object):
 sumatra.projects.get_diff_formatter = lambda: MockDiffFormatter
 
 class MockWorkingCopy(object):
-        def has_changed(self):
-            return False
-        def use_latest_version(self):
-            pass
-        def current_version(self):
-            return 999
-        def use_version(self, v):
-            pass
+    path = "/path/to/working/copy"
+    def has_changed(self):
+        return False
+    def use_latest_version(self):
+        pass
+    def current_version(self):
+        return 999
+    def use_version(self, v):
+        pass
+    def contains(self, path):
+        return True
         
 class MockRepository(object):
     url = "http://svn.example.com"
@@ -42,7 +45,9 @@ class MockRepository(object):
 
 class MockExecutable(object):
     name = "Python"
-    path = "/usr/local/bin/python"
+    path = sys.executable #"/usr/local/bin/python"
+    version = sys.version
+    requires_script = True
     def write_parameters(self, params, filename):
         pass
     def __getstate__(self):
