@@ -15,6 +15,7 @@ from sumatra.recordstore.django_store import DjangoRecordStore, db_config
 import test_recordstore
 import sumatra.projects
 
+
 def setup():
     global store, orig_load_project, Client
     import django.conf
@@ -30,19 +31,20 @@ def setup():
     Client = django.test.client.Client
     import django.test.utils
     django.test.utils.setup_test_environment()
-    
+
     #recordstore_setup()
     orig_load_project = sumatra.projects.load_project
     sumatra.projects.load_project = lambda: MockProject()
 
 def clean_up():
-    if os.path.exists("test.db"):
-        os.remove("test.db")    
+    for filename in ("test.db", "test2.db"):
+        if os.path.exists(filename):
+            os.remove(filename)
 
 def teardown():
     test_recordstore.teardown()
-    sumatra.projects.load_project = orig_load_project
     clean_up()
+    sumatra.projects.load_project = orig_load_project
 
 
 class MockProject(object):
@@ -58,15 +60,15 @@ def add_some_records():
         store.save(MockProject.name, r)
 
 class MockDataStore(object):
-    def get_content(self, key, path, max_length=100):
-        if path == "non_existent_file.txt":
+    def get_content(self, key, max_length=100):
+        if key.path == "non_existent_file.txt":
             raise IOError()
         else:
             return ""
 
 
 class TestWebInterface(unittest.TestCase):
-    
+
     def test_project_root(self):
         c = Client()
         response = c.get('/%s/' % MockProject.name)
@@ -78,7 +80,7 @@ class TestWebInterface(unittest.TestCase):
         c = Client()
         self.assertRaises(ObjectDoesNotExist,
                           c.get, '/%s/record0/' % MockProject.name)
- 
+
     def test__record_detail(self):
         add_some_records()
         c = Client()
@@ -89,39 +91,39 @@ class TestWebInterface(unittest.TestCase):
     def test__show_file_csv(self):
         sumatra.web.views.get_data_store = lambda t,p: MockDataStore()
         c = Client()
-        response = c.get("/%s/record1/datafile?path=test_file.csv" % MockProject.name)
+        response = c.get("/%s/record1/datafile?path=test_file.csv&digest=mock" % MockProject.name)
         assert 'show_csv.html' in [t.name for t in response.template]
-        
+
     def test__show_file_txt(self):
         sumatra.web.views.get_data_store = lambda t,p: MockDataStore()
         c = Client()
-        response = c.get("/%s/record1/datafile?path=test_file.txt" % MockProject.name)
+        response = c.get("/%s/record1/datafile?path=test_file.txt&digest=mock" % MockProject.name)
         assert 'show_file.html' in [t.name for t in response.template]
 
     def test__show_file_image(self):
         sumatra.web.views.get_data_store = lambda t,p: MockDataStore()
         c = Client()
-        response = c.get("/%s/record1/datafile?path=test_file.png" % MockProject.name)
+        response = c.get("/%s/record1/datafile?path=test_file.png&digest=mock" % MockProject.name)
         assert 'show_image.html' in [t.name for t in response.template]
 
     def test__show_file_other(self):
         sumatra.web.views.get_data_store = lambda t,p: MockDataStore()
         c = Client()
-        response = c.get("/%s/record1/datafile?path=test_file.doc" % MockProject.name)
+        response = c.get("/%s/record1/datafile?path=test_file.doc&digest=mock" % MockProject.name)
         assert 'show_file.html' in [t.name for t in response.template]
-        assert "Can't display" in response.context["content"] 
+        assert "Can't display" in response.context["content"]
 
     def test__show_nonexistent_file(self):
         sumatra.web.views.get_data_store = lambda t,p: MockDataStore()
         c = Client()
-        response = c.get("/%s/record1/datafile?path=non_existent_file.txt" % MockProject.name)
+        response = c.get("/%s/record1/datafile?path=non_existent_file.txt&digest=mock" % MockProject.name)
         assert 'show_file.html' in [t.name for t in response.template]
-        assert "File not found" in response.context["content"] 
+        assert "File not found" in response.context["content"]
 
     def test__show_image(self):
         sumatra.web.views.get_data_store = lambda t,p: MockDataStore()
         c = Client()
-        response = c.get("/%s/record1/image?path=test_file.jpg" % MockProject.name)
+        response = c.get("/%s/record1/image?path=test_file.jpg&digest=mock" % MockProject.name)
         self.assertEqual(response["Content-Type"], "image/jpeg")
 
     def test__show_diff(self):
@@ -131,6 +133,18 @@ class TestWebInterface(unittest.TestCase):
         assert 'show_diff.html' in [t.name for t in response.template]
 
 
+class TestFilters(unittest.TestCase):
+
+    def test__human_readable_duration(self):
+        from sumatra.web.templatetags import filters
+        self.assertEqual(filters.human_readable_duration(((6 * 60 + 32) * 60 + 12)), '6h 32m 12.00s')
+        self.assertEqual(filters.human_readable_duration((((8 * 24 + 7) * 60 + 6) * 60 + 5)), '8d 7h 6m 5.00s')
+        self.assertEqual(filters.human_readable_duration((((8 * 24 + 7) * 60 + 6) * 60)), '8d 7h 6m')
+        self.assertEqual(filters.human_readable_duration((((8 * 24 + 7) * 60) * 60)), '8d 7h')
+        self.assertEqual(filters.human_readable_duration((((8 * 24) * 60) * 60)), '8d')
+        self.assertEqual(filters.human_readable_duration((((8 * 24) * 60) * 60) + 0.12), '8d 0.12s')
+
+
 if __name__ == '__main__':
     setup()
     unittest.main()
